@@ -2,7 +2,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include <stdlib.h>
-#include "faad.h"
+#include "neaacdec.h"
 #include "mp4ff.h"
 #include "tagging.h"
 
@@ -26,7 +26,9 @@
 #define AAC_MAGIC     (unsigned char [4]) { 0xFF, 0xF9, 0x5C, 0x80 }
 
 static void        mp4_init(void);
+/*
 static void        mp4_about(void);
+*/
 static int         mp4_is_our_file(char *);
 static void        mp4_play(InputPlayback *);
 static void        mp4_stop(InputPlayback *);
@@ -47,7 +49,9 @@ InputPlugin mp4_ip =
 {
     .description = "MP4 Audio Plugin",
     .init = mp4_init,
+/*
     .about = mp4_about,
+*/
     .is_our_file = mp4_is_our_file,
     .play_file = mp4_play,
     .stop = mp4_stop,
@@ -76,6 +80,7 @@ static Mp4Config mp4cfg;
 static GThread   *decodeThread;
 GStaticMutex     mutex = G_STATIC_MUTEX_INIT;
 static int       seekPosition = -1;
+static volatile char pause_flag;
 
 void getMP4info(char*);
 int getAACTrack(mp4ff_t *);
@@ -100,6 +105,7 @@ static void mp4_init(void)
 {
     mp4cfg.file_type = FILE_UNKNOWN;
     seekPosition = -1;
+    pause_flag = 0;
     return;
 }
 
@@ -133,13 +139,15 @@ static void mp4_stop(InputPlayback *playback)
 /// aac_parse_frames needs a buffer at least 8 bytes long
 int aac_parse_frame(guchar *buf, int *srate, int *num)
 {
-        int i = 0, sr, fl = 0, id;
+        int i = 0, sr, fl = 0;
         static int srates[] = {96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000, 12000, 11025, 8000, 0, 0, 0};
 
         if((buf[i] != 0xFF) || ((buf[i+1] & 0xF6) != 0xF0))
                 return 0;
 
+/* We currently have no use for the id below.
         id = (buf[i+1] >> 3) & 0x01;    //id=1 mpeg2, 0: mpeg4
+*/
         sr = (buf[i+2] >> 2)  & 0x0F;
         if(sr > 11)
                 return 0;
@@ -275,9 +283,10 @@ static int mp4_is_our_fd(char *filename, VFSFile* file)
   return 0;
 }
 
+/* XXX TODO: Figure out cause of freaky symbol collision and resurrect
 static void mp4_about(void)
 {
-    static GtkWidget *aboutbox = NULL;
+    static GtkWidget aboutbox = NULL;
     gchar *about_text;
 
     about_text = g_strjoin ("", _("Using libfaad2-"), FAAD2_VERSION,
@@ -293,10 +302,11 @@ static void mp4_about(void)
                      G_CALLBACK(gtk_widget_destroyed), &aboutbox);
     g_free(about_text);
 }
+*/
 
 static void mp4_pause(InputPlayback *playback, short flag)
 {
-    playback->output->pause(flag);
+   pause_flag = flag;
 }
 
 static void mp4_seek(InputPlayback *data, int time)
@@ -344,7 +354,7 @@ static Tuple *mp4_get_song_tuple_base(char *filename, VFSFile *mp4fh)
         gint mp4track= getAACTrack(mp4file);
         gint numSamples = mp4ff_num_samples(mp4file, mp4track);
         guint framesize = 1024;
-        guint samplerate = 0;
+        gulong samplerate = 0;
         guchar channels = 0;
         gint msDuration;
         mp4AudioSpecificConfig mp4ASC;
@@ -478,7 +488,7 @@ static int my_decode_mp4( InputPlayback *playback, char *filename, mp4ff_t *mp4f
     mp4AudioSpecificConfig mp4ASC;
     guchar      *buffer = NULL;
     guint       bufferSize = 0;
-    guint       samplerate = 0;
+    gulong      samplerate = 0;
     guchar      channels = 0;
     gulong      msDuration;
     guint       numSamples;
@@ -542,6 +552,19 @@ static int my_decode_mp4( InputPlayback *playback, char *filename, mp4ff_t *mp4f
             seekPosition = -1;
         }
 
+        if (pause_flag) {
+           playback->output->pause (1);
+           while (pause_flag) {
+              if (seekPosition != -1) {
+                 playback->output->flush (seekPosition * 1000);
+                 sampleID = (long long) seekPosition * samplerate / (framesize - 1);
+                 seekPosition = -1;
+              }
+              g_usleep(50000);
+           }
+           playback->output->pause (0);
+        }
+
         /* Otherwise continue playing */
         buffer=NULL;
         bufferSize=0;
@@ -573,7 +596,6 @@ static int my_decode_mp4( InputPlayback *playback, char *filename, mp4ff_t *mp4f
         if((rc == 0) || (buffer== NULL) || (bufferSize == 0) || (bufferSize > BUFFER_SIZE)){
             g_print("MP4: read error\n");
             sampleBuffer = NULL;
-            sampleID=0;
             playback->output->buffer_free();
             playback->output->close_audio();
 
@@ -629,9 +651,11 @@ void my_decode_aac( InputPlayback *playback, char *filename, VFSFile *file )
     gulong      samplerate = 0;
     guchar      channels = 0;
     gulong      buffervalid = 0;
+    gulong	ret = 0;
     gchar       *ttemp = NULL, *stemp = NULL;
     gchar       *temp = g_strdup(filename);
     gchar       *xmmstitle = NULL;
+    gint        bitrate;
     gboolean    remote = aud_str_has_prefix_nocase(filename, "http:") ||
 			 aud_str_has_prefix_nocase(filename, "https:");
 
@@ -674,13 +698,21 @@ void my_decode_aac( InputPlayback *playback, char *filename, VFSFile *file )
     else
         xmmstitle = g_strdup(g_basename(temp));
 
+    ttemp = aud_vfs_get_metadata(file, "content-bitrate");
+    if (ttemp != NULL && *ttemp != '0')
+    {
+        bitrate = atoi(ttemp);
+        g_free(ttemp);
+    }
+    else
+        bitrate = -1;
+
     bufferconsumed = aac_probe(streambuffer, buffervalid);
     if(bufferconsumed) {
       buffervalid -= bufferconsumed;
       memmove(streambuffer, &streambuffer[bufferconsumed], buffervalid);
       buffervalid += aud_vfs_fread(&streambuffer[buffervalid], 1,
                      BUFFER_SIZE-buffervalid, file);
-      bufferconsumed = 0;
     }
 
     bufferconsumed = faacDecInit(decoder,
@@ -703,7 +735,7 @@ void my_decode_aac( InputPlayback *playback, char *filename, VFSFile *file )
         return;
     }
 
-    playback->set_params(playback, xmmstitle, -1, -1, samplerate, channels);
+    playback->set_params(playback, xmmstitle, -1, bitrate, samplerate, channels);
     playback->output->flush(0);
 
     while(buffer_playing && buffervalid > 0 && streambuffer != NULL)
@@ -716,10 +748,15 @@ void my_decode_aac( InputPlayback *playback, char *filename, VFSFile *file )
         {
             buffervalid -= bufferconsumed;
             memmove(streambuffer, &streambuffer[bufferconsumed], buffervalid);
-            buffervalid += aud_vfs_fread(&streambuffer[buffervalid], 1,
+            ret = aud_vfs_fread(&streambuffer[buffervalid], 1,
                          BUFFER_SIZE-buffervalid, file);
+            buffervalid += ret;
             bufferconsumed = 0;
 
+            /* XXX: buffer underrun on a shoutcast stream, well this is unpleasant. --nenolod */
+            if (ret == 0 && remote == TRUE)
+                break;
+
             ttemp = aud_vfs_get_metadata(file, "stream-name");
 
             if (ttemp != NULL)
@@ -741,7 +778,7 @@ void my_decode_aac( InputPlayback *playback, char *filename, VFSFile *file )
 
                     ostmp = stemp;
 
-                    playback->set_params(playback, xmmstitle, -1, -1, samplerate, channels);
+                    playback->set_params(playback, xmmstitle, -1, bitrate, samplerate, channels);
                 }
             }
 
