@@ -33,9 +33,7 @@
 extern "C"
 {
 #include <audacious/plugin.h>
-#include <audacious/configdb.h>
 #include <audacious/output.h>
-#include <audacious/util.h>
 }
 
 
@@ -139,6 +137,7 @@ print_left (const gchar * text)
   return GTK_WIDGET (label);
 }
 
+#if 0
 static void
 MessageBox (const char *title, const char *text, const char *button)
 {
@@ -158,6 +157,7 @@ MessageBox (const char *title, const char *text, const char *button)
   free (tmptxt);
   free (tmpbutton);
 }
+#endif
 
 /***** Dialog boxes *****/
 
@@ -912,29 +912,6 @@ adplug_is_our_fd (gchar * filename, VFSFile * fd)
   return FALSE;
 }
 
-static int
-adplug_is_our_file (char *filename)
-{
-  CSilentopl tmpopl;
-  VFSFile *fd = aud_vfs_buffered_file_new_from_uri (filename);
-  if (!fd)
-    return FALSE;
-
-  CPlayer *p = factory (fd, &tmpopl);
-
-  dbg_printf ("adplug_is_our_file(\"%s\"): returned ", filename);
-
-  if (p)
-  {
-    delete p;
-    dbg_printf ("TRUE\n");
-    return TRUE;
-  }
-
-  dbg_printf ("FALSE\n");
-  return FALSE;
-}
-
 static int
 adplug_get_time (InputPlayback * data)
 {
@@ -998,8 +975,10 @@ adplug_play (InputPlayback * data)
   // start player func
   dbg_printf ("play");
   plr.play_thread =  g_thread_self();
+  playback->playing = TRUE;
   playback->set_pb_ready(playback);
   play_loop(playback);
+  playback->playing = FALSE;
   dbg_printf (".\n");
 }
 
@@ -1036,7 +1015,7 @@ static void
 adplug_init (void)
 {
   dbg_printf ("adplug_init(): open, ");
-  ConfigDb *db = aud_cfg_db_open ();
+  mcs_handle_t *db = aud_cfg_db_open ();
 
   // Read configuration
   dbg_printf ("read, ");
@@ -1088,7 +1067,7 @@ static void
 adplug_quit (void)
 {
   dbg_printf ("adplug_quit(): open, ");
-  ConfigDb *db = aud_cfg_db_open ();
+  mcs_handle_t *db = aud_cfg_db_open ();
 
   // Close database
   dbg_printf ("db, ");
@@ -1124,6 +1103,13 @@ adplug_quit (void)
 
 /***** Plugin (exported) *****/
 
+const gchar *fmts[] =
+    { "a2m", "adl", "amd", "bam", "cff", "cmf", "d00", "dfm", "dmo", "dro",
+      "dtm", "hsc", "hsp", "ins", "jbm", "ksm", "laa", "lds", "m", "mad",
+      "mkj", "msc", "rad", "raw", "rix", "rol", "s3m", "sa2", "sat", "sci",
+      "sng", "wlf", "xad", "xsm",
+      NULL };
+
 InputPlugin adplug_ip = {
   NULL,                         // handle (filled by XMMS)
   NULL,                         // filename (filled by XMMS)
@@ -1133,13 +1119,12 @@ InputPlugin adplug_ip = {
   adplug_about,
   adplug_config,
   FALSE,
-  adplug_is_our_file,
-  NULL,                         // scan_dir (look in Input/cdaudio/cdaudio.c)
+  NULL,
+  NULL,
   adplug_play,
   adplug_stop,
   adplug_pause,
   adplug_seek,
-  NULL,                         // set_eq
   adplug_get_time,
   NULL,                         // get_volume (handled by output plugin)
   NULL,                         // set_volume (...)
@@ -1149,12 +1134,9 @@ InputPlugin adplug_ip = {
   NULL,                         // set_info_text (filled by XMMS)
   adplug_song_info,
   adplug_info_box,
-  NULL,                         // output plugin (filled by XMMS)
   adplug_get_tuple,
-  NULL,
-  NULL,
   adplug_is_our_fd,
-  NULL,
+  (gchar **)fmts,
 };
 
 InputPlugin *adplug_iplist[] = { &adplug_ip, NULL };
