@@ -2,14 +2,14 @@
 #define _CDNG_CONFIGURE_H
 #include <glib.h>
 
-extern struct cdng_cfg_t {
+typedef struct cdng_cfg_t {
 
 	gboolean	use_dae;
 	gboolean	use_cdtext;
 	gboolean	use_cddb;
-	gboolean	debug;
 	gchar		*device;
 	gchar		*cddb_server;
+	gchar		*cddb_path;
 	gint		cddb_port;
 	gboolean	cddb_http;
 	gint		limitspeed;
@@ -19,7 +19,7 @@ extern struct cdng_cfg_t {
 	gchar		*proxy_username;
 	gchar		*proxy_password;
 
-} cdng_cfg;
+} cdng_cfg_t;
 
 
 void	configure_create_gui(void);
