@@ -25,7 +25,6 @@
 #include <esd.h>
 
 #include <unistd.h>
-#include <audacious/util.h>
 
 #include "esdout.h"
 
@@ -158,6 +157,9 @@ esdout_setup_format(AFormat fmt, gint rate, gint nch)
     case FMT_S16_NE:
         esd_format = ESD_BITS16;
         break;
+    default:
+        /* for warning suppression */
+        break;
     }
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
@@ -408,7 +410,8 @@ esdout_open(AFormat fmt, gint rate, gint nch)
         g_free(esd_cfg.hostname);
     if (esd_cfg.use_remote)
         esd_cfg.hostname =
-            g_strdup_printf("%s:%d", esd_cfg.server, esd_cfg.port);
+            g_strdup_printf((strchr(esd_cfg.server, ':')) ? "[%s]:%d" : "%s:%d",
+                    esd_cfg.server, esd_cfg.port);
     else
         esd_cfg.hostname = NULL;
 
