@@ -17,11 +17,12 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "filewriter.h"
 #include "plugins.h"
+#include "convert.h"
 
 struct format_info input;
 
@@ -61,7 +62,7 @@ static gchar *fileext_str[] =
 #endif
 };
 
-static FileWriter plugin;
+static FileWriter *plugin;
 
 static GtkWidget *saveplace_hbox, *saveplace;
 static gboolean save_original = TRUE;
@@ -77,10 +78,15 @@ static gboolean prependnumber = FALSE;
 
 static gchar *file_path = NULL;
 
-static void file_init(void);
+VFSFile *output_file = NULL;
+guint64 offset = 0;
+Tuple *tuple = NULL;
+
+static OutputPluginInitStatus file_init(void);
 static void file_about(void);
 static gint file_open(AFormat fmt, gint rate, gint nch);
 static void file_write(void *ptr, gint length);
+static gint file_write_output(void *ptr, gint length);
 static void file_close(void);
 static void file_flush(gint time);
 static void file_pause(short p);
@@ -93,6 +99,7 @@ static void file_configure(void);
 OutputPlugin file_op =
 {
     .description = "FileWriter Plugin",
+    .probe_priority = 0,
     .init = file_init,
     .about = file_about,
     .configure = file_configure,
@@ -111,51 +118,48 @@ OutputPlugin *file_oplist[] = { &file_op, NULL };
 
 SIMPLE_OUTPUT_PLUGIN(filewriter, file_oplist);
 
+FileWriter *plugins[FILEEXT_MAX] = {
+    &wav_plugin,
+#ifdef FILEWRITER_MP3
+    &mp3_plugin,
+#endif
+#ifdef FILEWRITER_VORBIS
+    &vorbis_plugin,
+#endif
+#ifdef FILEWRITER_FLAC
+    &flac_plugin,
+#endif
+};
+
 static void set_plugin(void)
 {
     if (fileext < 0 || fileext >= FILEEXT_MAX)
         fileext = 0;
 
-    if (fileext == WAV)
-        plugin = wav_plugin;
-#ifdef FILEWRITER_MP3
-    if (fileext == MP3)
-        plugin = mp3_plugin;
-#endif
-#ifdef FILEWRITER_VORBIS
-    if (fileext == VORBIS)
-        plugin = vorbis_plugin;
-#endif
-#ifdef FILEWRITER_FLAC
-    if (fileext == FLAC)
-        plugin = flac_plugin;
-#endif
+    plugin = plugins[fileext];
 }
 
-static void file_init(void)
+static OutputPluginInitStatus file_init(void)
 {
     ConfigDb *db;
-    /*GtkWidget *menu_root;
-
-    menu_root = gtk_menu_item_new_with_label(_("FileWriter"));
-    gtk_widget_show(menu_root);
-    audacious_menu_plugin_item_add(AUDACIOUS_MENU_PLAYLIST_RCLICK, menu_root);*/
 
     db = aud_cfg_db_open();
-    aud_cfg_db_get_int(db, "filewriter", "fileext", &fileext);
-    aud_cfg_db_get_string(db, "filewriter", "file_path", &file_path);
-    aud_cfg_db_get_bool(db, "filewriter", "save_original", &save_original);
-    aud_cfg_db_get_bool(db, "filewriter", "use_suffix", &use_suffix);
-    aud_cfg_db_get_bool(db, "filewriter", "filenamefromtags", &filenamefromtags);
-    aud_cfg_db_get_bool(db, "filewriter", "prependnumber", &prependnumber);
+    aud_cfg_db_get_int(db, FILEWRITER_CFGID, "fileext", &fileext);
+    aud_cfg_db_get_string(db, FILEWRITER_CFGID, "file_path", &file_path);
+    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "save_original", &save_original);
+    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "use_suffix", &use_suffix);
+    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "filenamefromtags", &filenamefromtags);
+    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "prependnumber", &prependnumber);
     aud_cfg_db_close(db);
 
     if (!file_path)
         file_path = g_strdup(g_get_home_dir());
 
     set_plugin();
-    if (plugin.init)
-        plugin.init();
+    if (plugin->init)
+        plugin->init(&file_write_output);
+
+    return OUTPUT_PLUGIN_INIT_NO_DEVICES;
 }
 
 void file_about(void)
@@ -179,7 +183,7 @@ void file_about(void)
                                "\n"
                                "You should have received a copy of the GNU General Public License\n"
                                "along with this program; if not, write to the Free Software\n"
-                               "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,\n"
+                               "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,\n"
                                "USA."), _("Ok"), FALSE, NULL, NULL);
     gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                        GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
@@ -210,7 +214,7 @@ static gint file_open(AFormat fmt, gint rate, gint nch)
     {
         gchar *utf8 = aud_tuple_formatter_make_title_string(tuple, aud_get_gentitle_format());
 
-        g_strchomp(utf8); //chop trailing ^J --yaz
+        g_strchomp(utf8); /* chop trailing ^J --yaz */
 
         filename = g_locale_from_utf8(utf8, -1, NULL, NULL, NULL);
         g_free(utf8);
@@ -255,84 +259,34 @@ static gint file_open(AFormat fmt, gint rate, gint nch)
     if (!output_file)
         return 0;
 
-    rv = (plugin.open)();
+    convert_init(fmt, plugin->format_required, nch);
+
+    rv = (plugin->open)();
 
     return rv;
 }
 
-static void convert_buffer(gpointer buffer, gint length)
-{
-    gint i;
-
-    if (input.format == FMT_S8)
-    {
-        guint8 *ptr1 = buffer;
-        gint8 *ptr2 = buffer;
-
-        for (i = 0; i < length; i++)
-            *(ptr1++) = *(ptr2++) ^ 128;
-    }
-    if (input.format == FMT_S16_BE)
-    {
-        gint16 *ptr = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr++)
-            *ptr = GUINT16_SWAP_LE_BE(*ptr);
-    }
-    if (input.format == FMT_S16_NE)
-    {
-        gint16 *ptr = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr++)
-            *ptr = GINT16_TO_LE(*ptr);
-    }
-    if (input.format == FMT_U16_BE)
-    {
-        gint16 *ptr1 = buffer;
-        guint16 *ptr2 = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr2++)
-            *(ptr1++) = GINT16_TO_LE(GUINT16_FROM_BE(*ptr2) ^ 32768);
-    }
-    if (input.format == FMT_U16_LE)
-    {
-        gint16 *ptr1 = buffer;
-        guint16 *ptr2 = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr2++)
-            *(ptr1++) = GINT16_TO_LE(GUINT16_FROM_LE(*ptr2) ^ 32768);
-    }
-    if (input.format == FMT_U16_NE)
-    {
-        gint16 *ptr1 = buffer;
-        guint16 *ptr2 = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr2++)
-            *(ptr1++) = GINT16_TO_LE((*ptr2) ^ 32768);
-    }
-}
-
 static void file_write(void *ptr, gint length)
 {
-    if (input.format == FMT_S8 || input.format == FMT_S16_BE ||
-        input.format == FMT_U16_LE || input.format == FMT_U16_BE ||
-        input.format == FMT_U16_NE)
-        convert_buffer(ptr, length);
-#ifdef WORDS_BIGENDIAN
-    if (input.format == FMT_S16_NE)
-        convert_buffer(ptr, length);
-#endif
+    int len;
 
-    plugin.write(ptr, length);
+    len = convert_process(ptr, length);
+
+    plugin->write(convert_output, len);
+}
+
+static gint file_write_output(void *ptr, gint length)
+{
+    return aud_vfs_fwrite(ptr, 1, length, output_file);
 }
 
 static void file_close(void)
 {
-    plugin.close();
+    plugin->close();
+    convert_free();
 
     if (output_file)
     {
-        written = 0;
         aud_vfs_fclose(output_file);
     }
     output_file = NULL;
@@ -355,17 +309,17 @@ static void file_pause(short p)
 
 static gint file_free(void)
 {
-    return plugin.free();
+    return plugin->free();
 }
 
 static gint file_playing(void)
 {
-    return plugin.playing();
+    return plugin->playing();
 }
 
 static gint file_get_written_time(void)
 {
-    return plugin.get_written_time();
+    return plugin->get_written_time();
 }
 
 static gint file_get_output_time(void)
@@ -389,12 +343,12 @@ static void configure_ok_cb(gpointer data)
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prependnumber_toggle));
 
     db = aud_cfg_db_open();
-    aud_cfg_db_set_int(db, "filewriter", "fileext", fileext);
-    aud_cfg_db_set_string(db, "filewriter", "file_path", file_path);
-    aud_cfg_db_set_bool(db, "filewriter", "save_original", save_original);
-    aud_cfg_db_set_bool(db, "filewriter", "filenamefromtags", filenamefromtags);
-    aud_cfg_db_set_bool(db, "filewriter", "use_suffix", use_suffix);
-    aud_cfg_db_set_bool(db, "filewriter", "prependnumber", prependnumber);
+    aud_cfg_db_set_int(db, FILEWRITER_CFGID, "fileext", fileext);
+    aud_cfg_db_set_string(db, FILEWRITER_CFGID, "file_path", file_path);
+    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "save_original", save_original);
+    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "filenamefromtags", filenamefromtags);
+    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "use_suffix", use_suffix);
+    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "prependnumber", prependnumber);
 
     aud_cfg_db_close(db);
 
@@ -407,14 +361,16 @@ static void fileext_cb(GtkWidget *combo, gpointer data)
 {
     fileext = gtk_combo_box_get_active(GTK_COMBO_BOX(fileext_combo));
     set_plugin();
+    if (plugin->init)
+        plugin->init(&file_write_output);
 
-    gtk_widget_set_sensitive(plugin_button, plugin.configure != NULL);
+    gtk_widget_set_sensitive(plugin_button, plugin->configure != NULL);
 }
 
 static void plugin_configure_cb(GtkWidget *button, gpointer data)
 {
-    if (plugin.configure)
-        plugin.configure();
+    if (plugin->configure)
+        plugin->configure();
 }
 
 
@@ -509,7 +465,7 @@ static void file_configure(void)
         g_signal_connect(G_OBJECT(fileext_combo), "changed", G_CALLBACK(fileext_cb), NULL);
 
         plugin_button = gtk_button_new_with_label(_("Configure"));
-        gtk_widget_set_sensitive(plugin_button, plugin.configure != NULL);
+        gtk_widget_set_sensitive(plugin_button, plugin->configure != NULL);
         g_signal_connect(G_OBJECT(plugin_button), "clicked", G_CALLBACK(plugin_configure_cb), NULL);
         gtk_box_pack_end(GTK_BOX(fileext_hbox), plugin_button, FALSE, FALSE, 0);
 
@@ -630,8 +586,3 @@ static void file_configure(void)
         gtk_widget_show_all(configure_win);
     }
 }
-
-VFSFile *output_file = NULL;
-guint64 written = 0;
-guint64 offset = 0;
-Tuple *tuple = NULL;
