@@ -14,11 +14,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+/* #define FLACNG_DEBUG */
 
 #include "flacng.h"
-#include <audacious/util.h>
 #include <audacious/output.h>
 #include <audacious/i18n.h>
 #include "tools.h"
@@ -27,7 +27,7 @@
 #include "version.h"
 #include "debug.h"
 
-static gchar *flac_fmts[] = { "flac", NULL };
+static gchar *flac_fmts[] = { "flac", "fla", NULL };
 
 InputPlugin flac_ip = {
     .description = "FLACng Audio Plugin",
@@ -38,6 +38,7 @@ InputPlugin flac_ip = {
     .play_file = flac_play_file,
     .stop = flac_stop,
     .pause = flac_pause,
+    .mseek = flac_mseek,
     .seek = flac_seek,
     .get_song_info = flac_get_song_info,
     .get_song_tuple = flac_get_song_tuple,	// get a tuple
@@ -54,7 +55,8 @@ FLAC__StreamDecoder* main_decoder;
 callback_info* test_info;
 callback_info* main_info;
 gboolean plugin_initialized = FALSE;
-gint seek_to = -1;
+glong seek_to = -1;
+static volatile char pause_flag = 0;
 static GThread* thread = NULL;
 
 /* === */
@@ -216,9 +218,10 @@ gboolean flac_is_our_fd(gchar* filename, VFSFile* fd) {
         _LEAVE FALSE;
     }
 
-    if ((16 != test_info->stream.bits_per_sample) &&
+    if ((8  != test_info->stream.bits_per_sample) &&
+        (16 != test_info->stream.bits_per_sample) &&
         (24 != test_info->stream.bits_per_sample) &&
-        (8 != test_info->stream.bits_per_sample)) {
+        (32 != test_info->stream.bits_per_sample)) {
         _ERROR("This number of bits (%d) is currently not supported, stream not handled by this plugin",
             test_info->stream.bits_per_sample);
         INFO_UNLOCK(test_info);
@@ -264,12 +267,7 @@ gboolean flac_is_our_file(gchar* filename) {
 
 /* --- */
 
-void squeeze_audio(gint32* src, void* dst, guint count, guint src_res, guint dst_res) {
-
-    /*
-     * Changes the sample width of count samples in src from
-     * src_res to dst_res and places the result in dst
-     */
+void squeeze_audio(gint32* src, void* dst, guint count, guint res) {
 
     gint i;
     gint32* rp = src;
@@ -279,68 +277,25 @@ void squeeze_audio(gint32* src, void* dst, guint count, guint src_res, guint dst
 
     _ENTER;
 
-    _DEBUG("Converting %d samples %d->%d", count, src_res, dst_res);
+    _DEBUG("Converting %d samples to %d bit", count, res);
 
-    if ((src_res % 8 != 0) || (dst_res % 8 != 0)) {
-        _ERROR("Can not convert from %d bps to %d bps: not a multiple of 8",
-                src_res, dst_res);
+    if (res % 8 != 0) {
+        _ERROR("Can not convert to %d bps: not a multiple of 8", res);
         _LEAVE;
     }
 
-    if (16 == dst_res) {
-        if (8 == src_res) {
-            for (i=0; i<count; i++, wp2++, rp++) {
-                *wp2 = (*rp << 8) & 0xffff;
-            }
-        } else if (16 == src_res) {
-            for (i=0; i<count; i++, wp2++, rp++) {
-                *wp2 = *rp & 0xffff;
-            }
-        } else if (24 == src_res) {
-            for (i=0; i<count; i++, wp2++, rp++) {
-                *wp2 = (*rp >> 8) & 0xffff;
-            }
-        } else if (32 == src_res) {
-            for (i=0; i<count; i++, wp2++, rp++) {
-                *wp2 = (*rp >> 16) & 0xffff;
-            }
+    if (res == 8) {
+        for (i=0; i<count; i++, wp++, rp++) {
+            *wp = *rp & 0xff;
         }
-    } else if (8 == dst_res) {
-        if (8 == src_res) {
-            for (i=0; i<count; i++, wp++, rp++) {
-                *wp = *rp & 0xff;
-            }
-        } else if (16 == src_res) {
-            for (i=0; i<count; i++, wp++, rp++) {
-                *wp = (*rp >> 8) & 0xff;
-            }
-        } else if (24 == src_res) {
-            for (i=0; i<count; i++, wp++, rp++) {
-                *wp = (*rp >> 16) & 0xff;
-            }
-        } else if (32 == src_res) {
-            for (i=0; i<count; i++, wp++, rp++) {
-                *wp = (*rp >> 24) & 0xff;
-            }
-        }
-    } else if (32 == dst_res) {
-        if (8 == src_res) {
-            for (i=0; i<count; i++, wp4++, rp++) {
-                *wp4 = (*rp << 24) & 0xffffffff;
-            }
-        } else if (16 == src_res) {
-            for (i=0; i<count; i++, wp4++, rp++) {
-                *wp4 = (*rp << 16) & 0xffffffff;
-            }
-        } else if (24 == src_res) {
-            for (i=0; i<count; i++, wp4++, rp++) {
-                *wp4 = (*rp << 8) & 0xffffffff;
-            }
-        } else if (32 == src_res) {
-            for (i=0; i<count; i++, wp4++, rp++) {
-                *wp4 = *rp;
-            }
+    } else if (res == 16) {
+        for (i=0; i<count; i++, wp2++, rp++) {
+            *wp2 = *rp & 0xffff;
         }
+    } else if (res == 24 || res == 32) { /* 24bit value stored in lowest 3 bytes */
+       for (i=0; i<count; i++, wp4++, rp++) {
+           *wp4 = *rp;
+       }
     }
 
     _LEAVE;
@@ -348,6 +303,22 @@ void squeeze_audio(gint32* src, void* dst, guint count, guint src_res, guint dst
 
 /* --- */
 
+static void do_seek (InputPlayback * playback) {
+   playback->output->flush (seek_to);
+   FLAC__stream_decoder_seek_absolute (main_decoder, (long long) seek_to * main_info->stream.samplerate / 1000);
+   seek_to = -1;
+}
+
+static void do_pause (InputPlayback * playback) {
+   playback->output->pause (1);
+   while (pause_flag) {
+      if (seek_to != -1)
+         do_seek (playback);
+      g_usleep(50000);
+   }
+   playback->output->pause (0);
+}
+
 static gpointer flac_play_loop(gpointer arg) {
 
     /*
@@ -358,7 +329,6 @@ static gpointer flac_play_loop(gpointer arg) {
 
     gint32* read_pointer;
     gint elements_left;
-    gint seek_sample;
     FLAC__StreamDecoderState state;
     struct stream_info stream_info;
     guint sample_count;
@@ -377,9 +347,12 @@ static gpointer flac_play_loop(gpointer arg) {
     stream_info.channels = main_info->stream.channels;
     main_info->metadata_changed = FALSE;
 
-    if (!playback->output->open_audio(FMT_S16_NE,
-        main_info->stream.samplerate,
-        main_info->stream.channels)) {
+    ReplayGainInfo rg_info = get_replay_gain(main_info);
+    playback->set_replaygain_info(playback, &rg_info);
+
+    if (!playback->output->open_audio(SAMPLE_FMT(main_info->stream.bits_per_sample),
+                                      main_info->stream.samplerate,
+                                      main_info->stream.channels)) {
         playback->playing = FALSE;
         _ERROR("Could not open output plugin!");
         _LEAVE NULL;
@@ -454,17 +427,16 @@ static gpointer flac_play_loop(gpointer arg) {
 
             sample_count = MIN(OUTPUT_BLOCK_SIZE, elements_left);
 
-            /*
-             * Convert the audio.
-             * Currently this is hardcoded to 16 bit.
-             * 8 and 24 bit are sampled up/down linear.
-             */
-            _DEBUG("Converting %d samples to FMT_S16_NE", sample_count);
-            squeeze_audio(read_pointer, play_buffer, sample_count, main_info->frame.bits_per_sample, 16);
+            squeeze_audio(read_pointer, play_buffer, sample_count, main_info->stream.bits_per_sample);
 
             _DEBUG("Copying %d samples to output plugin", sample_count);
 
-            playback->pass_audio(playback, FMT_S16_NE, main_info->frame.channels, sample_count * sizeof(gint16), play_buffer, NULL);
+            playback->pass_audio(playback,
+                                 SAMPLE_FMT(main_info->stream.bits_per_sample),
+                                 main_info->stream.channels,
+                                 sample_count * SAMPLE_SIZE(main_info->stream.bits_per_sample),
+                                 play_buffer,
+                                 NULL);
 
             read_pointer += sample_count;
             elements_left -= sample_count;
@@ -479,24 +451,10 @@ static gpointer flac_play_loop(gpointer arg) {
         main_info->buffer_free = BUFFER_SIZE_SAMP;
         main_info->buffer_used = 0;
 
-        /*
-         * Do we have to seek to somewhere?
-         */
-        if (-1 != seek_to) {
-            _DEBUG("Seek requested to %d seconds", seek_to);
-
-            seek_sample = seek_to * main_info->stream.samplerate;
-            _DEBUG("Seek requested to sample %d", seek_sample);
-            if (FALSE == FLAC__stream_decoder_seek_absolute(main_decoder, seek_sample)) {
-                _ERROR("Could not seek to sample %d!", seek_sample);
-            } else {
-                /*
-                 * Flush the buffers
-                 */
-                flac_ip.output->flush(seek_to * 1000);
-            }
-            seek_to = -1;
-        }
+        if (seek_to != -1)
+	    do_seek (playback);
+        if (pause_flag)
+            do_pause (playback);
 
         /*
          * Have we reached the end of the stream?
@@ -509,7 +467,7 @@ static gpointer flac_play_loop(gpointer arg) {
 
             _DEBUG("End of stream reached, draining output buffer");
 
-            while((-1 == seek_to) && flac_ip.output->buffer_playing() && playback->playing == TRUE) {
+            while((-1 == seek_to) && playback->output->buffer_playing() && playback->playing == TRUE) {
                 g_usleep(40000);
             }
 
@@ -525,7 +483,7 @@ static gpointer flac_play_loop(gpointer arg) {
      */
     playback->playing = FALSE;
     _DEBUG("Closing audio device");
-    flac_ip.output->close_audio();
+    playback->output->close_audio();
     _DEBUG("Audio device closed");
 
     free(play_buffer);
@@ -611,17 +569,12 @@ void flac_stop(InputPlayback* input) {
 /* --- */
 
 void flac_pause(InputPlayback* input, gshort p) {
-
-    _ENTER;
-
-    input->output->pause(p);
-
-    _LEAVE;
+   pause_flag = p;
 }
 
 /* --- */
 
-void flac_seek(InputPlayback* input, gint time) {
+void flac_mseek(InputPlayback* input, gulong millisecond) {
 
     _ENTER;
 
@@ -630,8 +583,8 @@ void flac_seek(InputPlayback* input, gint time) {
         _LEAVE;
     }
 
-    _DEBUG("Requesting seek to %d", time);
-    seek_to = time;
+    _DEBUG("Requesting seek to %d", millisecond);
+    seek_to = millisecond;
 
     while (-1 != seek_to) {
         g_usleep(10000);
@@ -640,6 +593,11 @@ void flac_seek(InputPlayback* input, gint time) {
     _LEAVE;
 }
 
+void flac_seek(InputPlayback* input, gint time) {
+    gulong millisecond = time * 1000;
+    flac_mseek(input, millisecond);
+}
+
 /* --- */
 
 void flac_get_song_info(gchar* filename, gchar** title, gint* length) {
