@@ -9,11 +9,7 @@
 
 #include <audacious/plugin.h>
 #include <audacious/ui_preferences.h>
-#include <audacious/playlist.h>
-#include <audacious/configdb.h>
 #include <audacious/hook.h>
-#include <audacious/strings.h>
-#include <audacious/main.h>
 
 #include <unistd.h>
 #include <stdio.h>
@@ -23,6 +19,8 @@
 #include <wchar.h>
 #include <sys/time.h>
 
+#include <curl/curl.h>
+
 #include "plugin.h"
 #include "scrobbler.h"
 #include "gerpok.h"
@@ -101,9 +99,9 @@ static void aud_hook_playback_end(gpointer aud_hook_data, gpointer user_data)
 }
 
 void start(void) {
-	char *username = NULL, *password = NULL;
+	char *username = NULL, *password = NULL, *sc_url = NULL;
 	char *ge_username = NULL, *ge_password = NULL;
-	ConfigDb *cfgfile;
+	mcs_handle_t *cfgfile;
 	sc_going = 1;
 	ge_going = 1;
 	GError **moo = NULL;
@@ -113,6 +111,8 @@ void start(void) {
 				&username);
 		aud_cfg_db_get_string(cfgfile, "audioscrobbler", "password",
 				&password);
+		aud_cfg_db_get_string(cfgfile, "audioscrobbler", "sc_url",
+				&sc_url);		
 		aud_cfg_db_get_string(cfgfile, "audioscrobbler", "ge_username",
 				&ge_username);
 		aud_cfg_db_get_string(cfgfile, "audioscrobbler", "ge_password",
@@ -128,10 +128,11 @@ void start(void) {
 	}
 	else
 	{
-		sc_init(username, password);
+		sc_init(username, password, sc_url);
 
 		g_free(username);
 		g_free(password);
+		g_free(sc_url);
 	}
 	
 	if ((!ge_username || !ge_password) || (!*ge_username || !*ge_password))
@@ -342,6 +343,39 @@ static void *hs_thread(void *data __attribute__((unused)))
 	return NULL;
 }
 
+void setup_proxy(CURL *curl)
+{
+    mcs_handle_t *db;
+    gboolean use_proxy = FALSE;
+
+    db = aud_cfg_db_open();
+    aud_cfg_db_get_bool(db, NULL, "use_proxy", &use_proxy);
+    if (use_proxy == FALSE)
+    {
+        curl_easy_setopt(curl, CURLOPT_PROXY, "");
+    }
+    else
+    {
+        gchar *proxy_host = NULL, *proxy_port = NULL;
+        gboolean proxy_use_auth = FALSE;
+        aud_cfg_db_get_string(db, NULL, "proxy_host", &proxy_host);
+        aud_cfg_db_get_string(db, NULL, "proxy_port", &proxy_port);
+        curl_easy_setopt(curl, CURLOPT_PROXY, proxy_host);
+        curl_easy_setopt(curl, CURLOPT_PROXYPORT, atol(proxy_port));
+        aud_cfg_db_get_bool(db, NULL, "proxy_use_auth", &proxy_use_auth);
+        if (proxy_use_auth != FALSE)
+        {
+            gchar *userpwd = NULL, *user = NULL, *pass = NULL;
+            aud_cfg_db_get_string(db, NULL, "proxy_user", &user);
+            aud_cfg_db_get_string(db, NULL, "proxy_pass", &pass);
+            userpwd = g_strdup_printf("%s:%s", user, pass);
+            curl_easy_setopt(curl, CURLOPT_PROXYUSERPWD, userpwd);
+            g_free(userpwd);
+        }
+    }
+    aud_cfg_db_close(db);
+}
+
 GeneralPlugin *scrobbler_gplist[] = { &scrobbler_gp, NULL };
 
 DECLARE_PLUGIN(scrobbler, NULL, NULL, NULL, NULL, NULL, scrobbler_gplist, NULL, NULL);
