@@ -54,11 +54,7 @@
 
 #include <audacious/plugin.h>
 #include <audacious/output.h>
-#include <audacious/util.h>
-#include <audacious/configdb.h>
-#include <audacious/main.h>
 #include <audacious/i18n.h>
-#include <audacious/strings.h>
 
 #include "vorbis.h"
 
@@ -98,7 +94,7 @@ ov_callbacks vorbis_callbacks_stream = {
     NULL
 };
 
-gchar *vorbis_fmts[] = { "ogg", "ogm", NULL };
+gchar *vorbis_fmts[] = { "ogg", "ogm", "oga", NULL };
 
 InputPlugin vorbis_ip = {
     .description = "Ogg Vorbis Audio Plugin",  /* description */
@@ -123,9 +119,12 @@ DECLARE_PLUGIN(vorbis, NULL, NULL, vorbis_iplist, NULL, NULL, NULL, NULL, NULL);
 static OggVorbis_File vf;
 
 static GThread *thread;
-static volatile int seekneeded = -1;
+static volatile int seekneeded;
+static volatile char pause_flag;
 static int samplerate, channels;
 GMutex *vf_mutex;
+static GMutex * control_mutex;
+static GCond * control_cond;
 
 gchar **vorbis_tag_encoding_list = NULL;
 static GtkWidget *about_window;
@@ -222,16 +221,6 @@ vorbis_jump_to_time(InputPlayback *playback, long time)
     g_mutex_unlock(vf_mutex);
 }
 
-static void
-do_seek(InputPlayback *playback)
-{
-    if (seekneeded != -1) {
-        vorbis_jump_to_time(playback, seekneeded);
-        seekneeded = -1;
-        playback->eof = FALSE;
-    }
-}
-
 #define PCM_FRAMES 1024
 #define PCM_BUFSIZE PCM_FRAMES*2
 
@@ -284,8 +273,8 @@ vorbis_play_loop(gpointer arg)
      * alert us what section we're currently decoding in case we
      * need to change playback settings at a section boundary
      */
- 
-   
+
+
     g_mutex_lock(vf_mutex);
     if (ov_open_callbacks(datasource, &vf, NULL, 0, aud_vfs_is_streaming(fd->fd) ? vorbis_callbacks_stream : vorbis_callbacks) < 0) {
         vorbis_callbacks.close_func(datasource);
@@ -309,7 +298,7 @@ vorbis_play_loop(gpointer arg)
     title = vorbis_generate_title(&vf, filename);
     vorbis_update_replaygain(&rg_info);
     playback->set_replaygain_info(playback, &rg_info);
-    
+
     vi = ov_info(&vf, -1);
 
     samplerate = vi->rate;
@@ -324,8 +313,6 @@ vorbis_play_loop(gpointer arg)
         goto play_cleanup;
     }
 
-    seekneeded = -1;
-
     /*
      * Note that chaining changes things here; A vorbis file may
      * be a mix of different channels, bitrates and sample rates.
@@ -333,39 +320,61 @@ vorbis_play_loop(gpointer arg)
      * using the ov_ interface.
      */
 
-    while (playback->playing) {
-       
-        if (playback->eof) {
-            g_usleep(20000);
-            continue;
+    while (playback->playing)
+    {
+        g_mutex_lock (control_mutex);
+
+        if (pause_flag)
+        {
+            playback->output->pause (1);
+
+            while (pause_flag)
+            {
+                if (seekneeded != -1)
+                {
+                    vorbis_jump_to_time (playback, seekneeded);
+                    seekneeded = -1;
+                }
+
+                g_cond_wait (control_cond, control_mutex);
+            }
+
+            playback->output->pause (0);
         }
 
         if (seekneeded != -1)
-            do_seek(playback);
+        {
+            vorbis_jump_to_time (playback, seekneeded);
+            seekneeded = -1;
+        }
+
+        g_mutex_unlock (control_mutex);
 
-        
         int current_section = last_section;
 
         g_mutex_lock(vf_mutex);
-        
-        bytes = ov_read_float(&vf, &pcm, PCM_FRAMES, &current_section);
-        
-        if (bytes > 0)
-            bytes = vorbis_interleave_buffer(pcm, bytes, channels, pcmout);
 
-        /*
-         * We got some sort of error. Bail.
-         */
-        if (bytes <= 0 && bytes != OV_HOLE) {
-           /*
-            * EOF
-            */
-            AUDDBG("EOF\n");
-            playback->playing = 0;
-            playback->eof = TRUE;
-            current_section = last_section;
+        bytes = ov_read_float(&vf, &pcm, PCM_FRAMES, &current_section);
+
+        if (bytes == OV_HOLE)
+        {
+            g_mutex_unlock (vf_mutex);
+            continue;
         }
 
+        if (bytes <= 0)
+        {
+            g_mutex_unlock (vf_mutex);
+
+            while (playback->output->buffer_playing ())
+                g_usleep (10000);
+
+            playback->eof = 1;
+            break;
+        }
+
+        bytes = vorbis_interleave_buffer (pcm, bytes, channels, pcmout);
+
         if (current_section <= last_section) {
             /*
              * The info struct is different in each section.  vf
@@ -384,7 +393,8 @@ vorbis_play_loop(gpointer arg)
             if (vi->rate != samplerate || vi->channels != channels) {
                 samplerate = vi->rate;
                 channels = vi->channels;
-                while(playback->output->buffer_playing()) g_usleep(50000);
+                while (playback->output->buffer_playing())
+                    g_usleep(1000);
                 playback->output->close_audio();
                 if (!playback->output->
                         open_audio(FMT_FLOAT, vi->rate, vi->channels)) {
@@ -403,14 +413,8 @@ vorbis_play_loop(gpointer arg)
 
         playback->pass_audio(playback, FMT_FLOAT, channels, bytes, pcmout, &playback->playing);
 
-        if (!playback->playing)
-            goto stop_processing;
-
-        if (seekneeded != -1)
-            do_seek(playback);
-
         stop_processing:
-   
+
         if (current_section <= last_section) {
             /*
             * set total play time, bitrate, rate, and channels of
@@ -426,33 +430,19 @@ vorbis_play_loop(gpointer arg)
                 time = ov_time_total(&vf, -1) * 1000;
 
             g_mutex_unlock(vf_mutex);
-        
+
             playback->set_params(playback, title, time, br, samplerate, channels);
 
             timercount = playback->output->output_time();
 
             last_section = current_section;
-         
+
         }
     } /* main loop */
 
-    if (!playback->error) {
-        /*this loop makes it not skip the last ~4 seconds, but the playback 
-         * timer isn't updated in this period, so it still needs a bit of work 
-         *
-         * majeru
-         */
-        if(playback->eof) /* do it only on EOF --asphyx */
-            while(playback->output->buffer_playing()) {
-                AUDDBG("waiting for empty output buffer\n");
-                g_usleep(50000);
-            }
+  play_cleanup:
+    playback->output->close_audio();
 
-        playback->output->close_audio();
-    }
-
-
-    play_cleanup:
     g_free(title);
 
     /*
@@ -472,6 +462,8 @@ vorbis_play(InputPlayback *playback)
     playback->playing = 1;
     playback->eof = 0;
     playback->error = FALSE;
+    seekneeded = -1;
+    pause_flag = 0;
 
     thread = g_thread_self();
     playback->set_pb_ready(playback);
@@ -492,16 +484,19 @@ vorbis_stop(InputPlayback *playback)
 static void
 vorbis_pause(InputPlayback *playback, short p)
 {
-    playback->output->pause(p);
+    g_mutex_lock (control_mutex);
+    pause_flag = p;
+    g_cond_broadcast (control_cond);
+    g_mutex_unlock (control_mutex);
 }
 
 static void
 vorbis_seek(InputPlayback *data, int time)
 {
+    g_mutex_lock (control_mutex);
     seekneeded = time;
-
-    while (seekneeded != -1)
-        g_usleep(20000);
+    g_cond_broadcast (control_cond);
+    g_mutex_unlock (control_mutex);
 }
 
 /* Make sure you've locked vf_mutex */
@@ -518,14 +513,14 @@ vorbis_update_replaygain(ReplayGainInfo *rg_info)
     rg_gain = vorbis_comment_query(comment, "replaygain_album_gain", 0);
     if (!rg_gain) rg_gain = vorbis_comment_query(comment, "rg_audiophile", 0);    /* Old */
     rg_info->album_gain = rg_gain != NULL ? atof(rg_gain) : 0.0;
-    
+
     rg_gain = vorbis_comment_query(comment, "replaygain_track_gain", 0);
     if (!rg_gain) rg_gain = vorbis_comment_query(comment, "rg_radio", 0);    /* Old */
     rg_info->track_gain = rg_gain != NULL ? atof(rg_gain) : 0.0;
-    
+
     rg_peak = vorbis_comment_query(comment, "replaygain_album_peak", 0);
     rg_info->album_peak = rg_peak != NULL ? atof(rg_peak) : 0.0;
-    
+
     rg_peak = vorbis_comment_query(comment, "replaygain_track_peak", 0);
     if (!rg_peak) rg_peak = vorbis_comment_query(comment, "rg_peak", 0);  /* Old */
     rg_info->track_peak = rg_peak != NULL ? atof(rg_peak) : 0.0;
@@ -599,7 +594,7 @@ get_aud_tuple_for_vorbisfile(OggVorbis_File * vorbisfile, gchar *filename)
     }
     else
         aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, "Ogg Vorbis");
-    
+
     aud_tuple_associate_string(tuple, FIELD_MIMETYPE, NULL, "application/ogg");
 
     return tuple;
@@ -707,7 +702,7 @@ vorbis_aboutbox(void)
 static void
 vorbis_init(void)
 {
-    ConfigDb *db;
+    mcs_handle_t *db;
     gchar *tmp = NULL;
 
     memset(&vorbis_cfg, 0, sizeof(vorbis_config_t));
@@ -751,6 +746,8 @@ vorbis_init(void)
     aud_cfg_db_close(db);
 
     vf_mutex = g_mutex_new();
+    control_mutex = g_mutex_new ();
+    control_cond = g_cond_new ();
 
     aud_mime_set_plugin("application/ogg", &vorbis_ip);
 }
@@ -790,6 +787,8 @@ vorbis_cleanup(void)
 
     g_strfreev(vorbis_tag_encoding_list);
     g_mutex_free(vf_mutex);
+    g_mutex_free (control_mutex);
+    g_cond_free (control_cond);
 }
 
 static size_t
