@@ -18,6 +18,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
+/* #define AUD_DEBUG 1 */
+
 #include <config.h>
 
 #include <glib.h>
@@ -31,11 +34,6 @@
 #include <sys/errno.h>
 
 #include <audacious/plugin.h>
-#include <audacious/main.h>
-#include <audacious/util.h>
-#include <audacious/strings.h>
-#include <audacious/playlist.h>
-#include <audacious/playlist_container.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -75,17 +73,13 @@ static const xspf_entry_t xspf_entries[] = {
     { FIELD_GENRE,        "genre",        TUPLE_STRING,   TRUE,   CMP_DEF },
     { FIELD_MTIME,        "mtime",        TUPLE_INT,      TRUE,   CMP_DEF },
     { FIELD_FORMATTER,    "formatter",    TUPLE_STRING,   TRUE,   CMP_DEF },
+    { FIELD_COPYRIGHT,    "copyright",    TUPLE_STRING,   TRUE,   CMP_DEF },
+    { FIELD_PERFORMER,    "performer",    TUPLE_STRING,   TRUE,   CMP_DEF },
+    { FIELD_SUBSONG_ID,   "subsong-id",   TUPLE_INT,      TRUE,   CMP_DEF },
+    { FIELD_SUBSONG_NUM,  "subsong-num",  TUPLE_INT,      TRUE,   CMP_DEF },
 };
 
-static const gint xspf_nentries = (sizeof(xspf_entries) / sizeof(xspf_entry_t));
-
-/* we need encoding conversion */
-#ifdef DEBUG
-#  define XSDEBUG(...) { g_print("xspf[%s:%d]: ", __FUNCTION__, (int) __LINE__); g_print(__VA_ARGS__); }
-#else
-#  define XSDEBUG(...) /* stub */
-#endif
-
+static const gint xspf_nentries = (sizeof(xspf_entries) / sizeof(xspf_entries[0]));
 
 static gboolean is_uri(gchar *uri)
 {
@@ -144,8 +138,10 @@ static void xspf_add_file(xmlNode *track, const gchar *filename,
             if (!xmlStrcmp(nptr->name, (xmlChar *)"location")) {
                 /* Location is a special case */
                 gchar *str = (gchar *)xmlNodeGetContent(nptr);
-
-                location = g_strdup_printf("%s%s", base ? base : "", str);
+                if (!strstr(str, "://") && base)
+                    location = g_strdup_printf("%s/%s", base, str);
+                else
+                    location = g_strdup(str);
                 xmlFree(str);
             } else {
                 /* Rest of the nodes are handled here */
@@ -171,6 +167,7 @@ static void xspf_add_file(xmlNode *track, const gchar *filename,
                             break;
                         
                         case TUPLE_INT:
+                            AUDDBG("field=%s val=%s\n", xspf_entries[i].xspfName, str);
                             aud_tuple_associate_int(tuple, xspf_entries[i].tupleField, NULL, atol((char *)str));
                             break;
                         
@@ -187,29 +184,24 @@ static void xspf_add_file(xmlNode *track, const gchar *filename,
     }
 
     if (location) {
-        gchar *realfn = NULL, *scratch = NULL;
+        gchar *scratch = NULL;
 
         /* filename and path in tuple must be unescaped. */
-        scratch = g_filename_from_uri(location, NULL, NULL);
-        realfn = aud_str_to_utf8(scratch ? scratch : location);
-        g_free(scratch);
-
-        scratch = g_path_get_basename(realfn);
+        scratch = aud_uri_to_display_basename(location);
         aud_tuple_associate_string(tuple, FIELD_FILE_NAME, NULL, scratch);
         g_free(scratch);
 
-        scratch = g_path_get_dirname(realfn);
+        scratch = aud_uri_to_display_dirname(location);
         aud_tuple_associate_string(tuple, FIELD_FILE_PATH, NULL, scratch);
         g_free(scratch);
 
-        aud_tuple_associate_string(tuple, FIELD_FILE_EXT, NULL, strrchr(realfn, '.'));
+        aud_tuple_associate_string(tuple, FIELD_FILE_EXT, NULL, strrchr(location, '.'));
 
-        XSDEBUG("tuple->file_name = %s\n", aud_tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
-        XSDEBUG("tuple->file_path = %s\n", aud_tuple_get_string(tuple, FIELD_FILE_PATH, NULL));
+        AUDDBG("tuple->file_name = %s\n", aud_tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
+        AUDDBG("tuple->file_path = %s\n", aud_tuple_get_string(tuple, FIELD_FILE_PATH, NULL));
 
         /* add file to playlist */
         aud_playlist_load_ins_file_tuple(playlist, location, filename, pos, tuple);
-        g_free(realfn);
         pos++;
     }
 
@@ -260,7 +252,7 @@ static void xspf_playlist_load(const gchar *filename, gint pos)
 
     g_return_if_fail(filename != NULL);
 
-    XSDEBUG("filename='%s', pos=%d\n", filename, pos);
+    AUDDBG("filename='%s', pos=%d\n", filename, pos);
 
     doc = xmlRecoverFile(filename);
     if (doc == NULL)
@@ -274,7 +266,7 @@ static void xspf_playlist_load(const gchar *filename, gint pos)
             
             base = (gchar *)xmlNodeGetBase(doc, nptr);
 
-            XSDEBUG("base @1 = %s\n", base);
+            AUDDBG("base #1 = %s\n", base);
             
             // if filename is specified as a base, ignore it.
             tmp = xmlURIUnescapeString(base, -1, NULL);
@@ -286,7 +278,12 @@ static void xspf_playlist_load(const gchar *filename, gint pos)
                 g_free(tmp);
             }
             
-            XSDEBUG("base @2 = %s\n", base);
+            AUDDBG("base #2 = %s\n", base);
+            
+            if (!base)
+                base = g_path_get_dirname(filename);
+            
+            AUDDBG("base #3 = %s\n", base);
             
             for (nptr2 = nptr->children; nptr2 != NULL; nptr2 = nptr2->next) {
 
@@ -360,7 +357,7 @@ static void xspf_playlist_save(const gchar *filename, gint pos)
     gchar *base = NULL;
     Playlist *playlist = aud_playlist_get_active();
 
-    XSDEBUG("filename='%s', pos=%d\n", filename, pos);
+    AUDDBG("filename='%s', pos=%d\n", filename, pos);
 
     doc = xmlNewDoc((xmlChar *)"1.0");
     doc->charset = XML_CHAR_ENCODING_UTF8;
@@ -376,7 +373,7 @@ static void xspf_playlist_save(const gchar *filename, gint pos)
     if (playlist->attribute & PLAYLIST_USE_RELATIVE) {
         /* prescan to determine base uri */
         for (node = playlist->entries; node != NULL; node = g_list_next(node)) {
-            gchar *ptr1, *ptr2;
+            gchar *ptr1, *ptr2, *ptrslash;
             PlaylistEntry *entry = PLAYLIST_ENTRY(node->data);
             gchar *tmp;
             gint tmplen = 0;
@@ -394,21 +391,24 @@ static void xspf_playlist_save(const gchar *filename, gint pos)
             }
             
             ptr1 = base;
-            ptr2 = tmp;
+            ptrslash = ptr2 = tmp;
 
             while(ptr1 && ptr2 && *ptr1 && *ptr2 && *ptr1 == *ptr2) {
+                if (*ptr2 == '/') ptrslash = ptr2 + 1;
+
                 ptr1++;
                 ptr2++;
             }
             
-            *ptr2 = '\0';       //terminate
-            tmplen = ptr2 - tmp;
+            if (!(*ptrslash)) ptrslash--;
+            *ptrslash = '\0';       //terminate
+            tmplen = ptrslash - tmp;
 
             if (tmplen <= baselen) {
                 g_free(base);
                 base = tmp;
                 baselen = tmplen;
-                XSDEBUG("base='%s', baselen=%d\n", base, baselen);
+                AUDDBG("base='%s', baselen=%d\n", base, baselen);
             } else
                 g_free(tmp);
         }
@@ -425,7 +425,7 @@ static void xspf_playlist_save(const gchar *filename, gint pos)
             }
 
             if (!is_uri(base)) {
-                XSDEBUG("base is not uri. something is wrong.\n");
+                AUDDBG("base is not uri. something is wrong.\n");
                 tmp = g_strdup_printf("file://%s", base);
                 xmlSetProp(rootnode, (xmlChar *)"xml:base", (xmlChar *)tmp);
                 g_free(tmp);
@@ -472,14 +472,14 @@ static void xspf_playlist_save(const gchar *filename, gint pos)
         location = xmlNewNode(NULL, (xmlChar *)"location");
 
         if (is_uri(entry->filename)) {   /* uri */
-            XSDEBUG("filename is uri\n");
+            AUDDBG("filename is uri\n");
             filename = g_strdup(entry->filename + baselen); // entry->filename is always uri now.
         } else {                  /* local file (obsolete) */
             gchar *tmp = (gchar *) xspf_path_to_uri((const xmlChar *)entry->filename + baselen);
             if (base) { /* relative */
                 filename = g_strdup_printf("%s", tmp);
             } else {
-                XSDEBUG("absolute and local (obsolete)\n");
+                AUDDBG("absolute and local (obsolete)\n");
                 filename = g_filename_to_uri(tmp, NULL, NULL);
             }
             g_free(tmp);
@@ -526,9 +526,6 @@ static void xspf_playlist_save(const gchar *filename, gint pos)
                     xspf_add_node(track, xs->type, xs->isMeta, xs->xspfName, scratch, scratchi);
             }
 
-            /* Write mtime unconditionally to support staticlist */
-            xspf_add_node(track, TUPLE_INT, TRUE, "mtime", NULL,
-                aud_tuple_get_int(entry->tuple, FIELD_MTIME, NULL));
         } else {
 
             if (entry->title != NULL && g_utf8_validate(entry->title, -1, NULL))
